#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

import sys
import traceback
from datetime import datetime

from airbyte_cdk.entrypoint import AirbyteEntrypoint, launch
from airbyte_cdk.models import AirbyteErrorTraceMessage, AirbyteMessage, AirbyteTraceMessage, TraceType, Type
from source_azure_blob_storage import Config, SourceAzureBlobStorage, SourceAzureBlobStorageStreamReader


def run():
    args = sys.argv[1:]
    catalog_path = AirbyteEntrypoint.extract_catalog(args)
    config_path = AirbyteEntrypoint.extract_config(args)
    state_path = AirbyteEntrypoint.extract_state(args)
    try:
        source = SourceAzureBlobStorage(
            SourceAzureBlobStorageStreamReader(),
            Config,
            SourceAzureBlobStorage.read_catalog(catalog_path) if catalog_path else None,
            SourceAzureBlobStorage.read_config(config_path) if catalog_path else None,
            SourceAzureBlobStorage.read_state(state_path) if catalog_path else None,
        )
    except Exception:
        print(
            AirbyteMessage(
                type=Type.TRACE,
                trace=AirbyteTraceMessage(
                    type=TraceType.ERROR,
                    emitted_at=int(datetime.now().timestamp() * 1000),
                    error=AirbyteErrorTraceMessage(
                        message="Error starting the sync. This could be due to an invalid configuration or catalog. Please contact Support for assistance.",
                        stack_trace=traceback.format_exc(),
                    ),
                ),
            ).json()
        )
    else:
        launch(source, args)



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

from typing import Any, Mapping

from airbyte_cdk.config_observation import emit_configuration_as_airbyte_control_message
from airbyte_cdk.sources.file_based.file_based_source import FileBasedSource

from .legacy_config_transformer import LegacyConfigTransformer


class SourceAzureBlobStorage(FileBasedSource):
    @classmethod
    def read_config(cls, config_path: str) -> Mapping[str, Any]:
        """
        Used to override the default read_config so that when the new file-based Azure Blob Storage connector processes a config
        in the legacy format, it can be transformed into the new config. This happens in entrypoint before we
        validate the config against the new spec.
        """
        config = FileBasedSource.read_config(config_path)
        if not cls._is_v1_config(config):
            converted_config = LegacyConfigTransformer.convert(config)
            emit_configuration_as_airbyte_control_message(converted_config)
            return converted_config
        return config

    @staticmethod
    def _is_v1_config(config: Mapping[str, Any]) -> bool:
        return "streams" in config



# Copyright (c) 2023 Airbyte, Inc., all rights reserved.

import logging
from contextlib import contextmanager
from io import IOBase
from typing import Iterable, List, Optional

import pytz
from airbyte_cdk.sources.file_based.file_based_stream_reader import AbstractFileBasedStreamReader, FileReadMode
from airbyte_cdk.sources.file_based.remote_file import RemoteFile
from azure.storage.blob import BlobServiceClient, ContainerClient
from smart_open import open

from .config import Config


class SourceAzureBlobStorageStreamReader(AbstractFileBasedStreamReader):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._config = None

    @property
    def config(self) -> Config:
        return self._config

    @config.setter
    def config(self, value: Config) -> None:
        self._config = value

    @property
    def account_url(self) -> str:
        if not self.config.azure_blob_storage_endpoint:
            return f"https://{self.config.azure_blob_storage_account_name}.blob.core.windows.net"
        return self.config.azure_blob_storage_endpoint

    @property
    def azure_container_client(self):
        return ContainerClient(
            self.account_url,
            container_name=self.config.azure_blob_storage_container_name,
            credential=self.config.azure_blob_storage_account_key,
        )

    @property
    def azure_blob_service_client(self):
        return BlobServiceClient(self.account_url, credential=self.config.azure_blob_storage_account_key)

    def get_matching_files(
        self,
        globs: List[str],
        prefix: Optional[str],
        logger: logging.Logger,
    ) -> Iterable[RemoteFile]:
        prefixes = [prefix] if prefix else self.get_prefixes_from_globs(globs)
        prefixes = prefixes or [None]
        for prefix in prefixes:
            for blob in self.azure_container_client.list_blobs(name_starts_with=prefix):
                remote_file = RemoteFile(uri=blob.name, last_modified=blob.last_modified.astimezone(pytz.utc).replace(tzinfo=None))
                if not globs or self.file_matches_globs(remote_file, globs):
                    yield remote_file

    def open_file(self, file: RemoteFile, mode: FileReadMode, encoding: Optional[str], logger: logging.Logger) -> IOBase:
        try:
            result = open(
                f"azure://{self.config.azure_blob_storage_container_name}/{file.uri}",
                transport_params={"client": self.azure_blob_service_client},
                mode=mode.value,
                encoding=encoding,
            )
        except OSError:
            logger.warning(
                f"We don't have access to {file.uri}. The file appears to have become unreachable during sync."
                f"Check whether key {file.uri} exists in `{self.config.azure_blob_storage_container_name}` container and/or has proper ACL permissions"
            )
        return result



#
# Copyright (c) 2023 Airbyte, Inc., all rights reserved.
#

from typing import Any, Mapping, MutableMapping


class LegacyConfigTransformer:
    """
    Class that takes in Azure Blob Storage source configs in the legacy format and transforms them into
    configs that can be used by the new Azure Blob Storage source built with the file-based CDK.
    """

    @classmethod
    def convert(cls, legacy_config: Mapping) -> MutableMapping[str, Any]:
        azure_blob_storage_blobs_prefix = legacy_config.get("azure_blob_storage_blobs_prefix", "")

        return {
            "azure_blob_storage_endpoint": legacy_config.get("azure_blob_storage_endpoint", None),
            "azure_blob_storage_account_name": legacy_config["azure_blob_storage_account_name"],
            "azure_blob_storage_account_key": legacy_config["azure_blob_storage_account_key"],
            "azure_blob_storage_container_name": legacy_config["azure_blob_storage_container_name"],
            "streams": [
                {
                    "name": legacy_config["azure_blob_storage_container_name"],
                    "legacy_prefix": azure_blob_storage_blobs_prefix,
                    "validation_policy": "Emit Record",
                    "format": {"filetype": "jsonl"},
                }
            ],
        }
